class TrainConfig(object):
    """
    Config in Training Mode
    """

    def __init__(self):
        self.lr = 0.0
        self.l2norm = 0.0
        self.epoch = 1
        self.batch_size = 1
        self.device = -1 # set >0 to use GPU


class ModelConfig(object):
    """
    Configuration in model setting
    """
    def __init__(self):
        self.input_size = 0
        self.hidden_size = 0
        self.embd_size = 0
        self.output_size = 0



class DataConfig(object):
    """
    Data Config
    """
    def __init__(self):
        self.kg = ""
        self.text = ""
        self.question = ""
        self.process = dict()

    def set_process(self, **kwargs):
        """
        Config process data
        """
        for k in kwargs.keys():
            self.process[k] = kwargs[k]


class Config(object):
    """
    Total Model and Training Config Class
    """

    def __init__(self, train_cfg: TrainConfig,
                 model_cfg: ModelConfig,
                 data_cfg: DataConfig):
        self.model = model_cfg
        self.train = train_cfg
        self.data = data_cfg



"""
logging in deep learning
"""

def logger(train_logs: dict, test_logs:dict, file:str="", **kwargs):
    """
    Args:
        logs: dict type, must have key "test" or "train"
        file: path file

    Returns: None

    """
    for k in kwargs.keys():
        print(f"{k} is {kwargs[k]}")
    print("---------------Train---------------")
    for k in train_logs.keys():
        print(f"Training {k} is {train_logs[k]}")
    print("---------------Test---------------")
    for k in test_logs.keys():
        print(f"Testing {k} is {test_logs[k]}")

    if len(file) != 0:
        with open(file,'w') as fp:
            for k in train_logs.keys():
                print(f"Training {k} is {train_logs[k]}", file=fp)
            for k in test_logs.keys():
                print(f"Testing {k} is {test_logs[k]}", file=fp)
    print("End logs")



"""
自然语言中数据预处理代码
"""
from gxdltk.data.header import *

__all__ = ["gen_vocab_file", "tokenized"]

def gen_vocab_file(text_data: str, key: str,
                   vocab_file: str = "vocab.txt",
                   spl="\t"):
    """
    TODO: Tokenized method
    TODO: Index by freq
    File Format
    First Line : vocab size
    Others: word spl id
    Parameters
    ----------
    vocab_file
    text_data: json format , json array of
    key: key of each data item
    spl: split in vocab.txt
    Returns
    -------

    """
    voc = set()
    with open(text_data) as textfp:
        text = json.load(textfp)
        for item in text:
            # item is dict type
            if isinstance(item, dict) and key in item.keys():
                text_item = item[key]
                text_item = text_item.split()
                voc = voc | set(text_item)

    with open(vocab_file, 'w') as voc_fp:
        print(f"{len(voc)}")
        for index, word in enumerate(voc):
            print(f"{word}{spl}{index}", file=voc_fp)

def tokenized(sent: str) -> list:
    raise NotImplementedError



from setuptools import setup
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()
setup(
    name='gxdltk',  # package name
    version='0.3-beta-5',  # version
    description="DeepLearning Toolkit for pytorch",
    author='gawainx',
    author_email='liangyixp@live.cn',
    install_requires=['numpy', 'torch', 'mxnet'],
    long_description=long_description,
    long_description_content_type="text/markdown",
    classifiers=[
            "Programming Language :: Python :: 3.7",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
)



"""
General Hooker for forward and backward
"""
from gxdltk.th.header import *

__all__ = ["observer"]


def observer(module, it, output):
    """
    Forward hook for watch all var size
    using Name tensor in torch1.3
    :param module: torch.nn.Module
    :param it: any type
    :param output: any type, model output
    """
    print(f"Module \n {module}")
    if isinstance(it, tensor):
        print(f"Input shape {it.shape}")
    elif isinstance(it, tuple):
        for i in range(len(it)):
            print(f"Input {i} shape {it[i].shape}")

    if isinstance(output, tensor):
        print(f"Input shape {output.shape}")
    elif isinstance(output, tuple):
        for i in range(len(output)):
            print(f"Input {i} shape {output[i].shape}")
